package com.emato.file.tunnel.config;

import com.emato.file.tunnel.config.properties.RabbitMQProperties;
import com.emato.file.tunnel.mq.callback.SimpleConfirmCallBack;
import com.emato.file.tunnel.mq.callback.SimpleReturnsCallBack;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.RabbitListenerContainerFactory;
import org.springframework.amqp.support.converter.MessageConversionException;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.amqp.RabbitProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author lhm
 * @version 1.0
 * 2021-08-24 16:04
 */
@Configuration
public class RabbitMQConfig {

    @Autowired
    private RabbitProperties rabbitProperties;

    @Autowired
    private RabbitMQProperties rabbitMQProperties;


    @Bean
    public ConnectionFactory connectionFactory () {
        CachingConnectionFactory connectionFactory = new CachingConnectionFactory();
        connectionFactory.setUsername(rabbitProperties.getUsername());
        connectionFactory.setPassword(rabbitProperties.getPassword());
        connectionFactory.setHost(rabbitProperties.getHost());
        connectionFactory.setVirtualHost(rabbitProperties.getVirtualHost());
        connectionFactory.setPort(rabbitProperties.getPort());
        // 共用同一个Channel
        connectionFactory.setCacheMode(CachingConnectionFactory.CacheMode.CHANNEL);
        // 获取配置的Channel缓存大小
        connectionFactory.setChannelCacheSize(rabbitProperties.getCache().getChannel().getSize());
        // 消息到达broke后触发回调
        connectionFactory.setPublisherConfirmType(CachingConnectionFactory.ConfirmType.CORRELATED);
        return connectionFactory;
    }

    /**
     * SimpleRabbitListenerContainerFactory发现消息中有content_type有text就会默认将其转换成string类型
     * @return rabbit监听器容器工厂
     */
    @Bean
    public RabbitListenerContainerFactory<?> rabbitListenerContainerFactory(ConnectionFactory connectionFactory) {
        SimpleRabbitListenerContainerFactory containerFactory = new SimpleRabbitListenerContainerFactory();
        containerFactory.setConnectionFactory(connectionFactory);
        containerFactory.setConcurrentConsumers(3);
        containerFactory.setMessageConverter(new MessageConverter() {
            @Override
            public Message toMessage(Object object, MessageProperties messageProperties) throws MessageConversionException {
                return new Message(object.toString().getBytes(), messageProperties);
            }

            @Override
            public Object fromMessage(Message message) throws MessageConversionException {
                return new String(message.getBody());
            }
        });
        containerFactory.setAcknowledgeMode(AcknowledgeMode.MANUAL);
        return containerFactory;
    }

    @Bean
    public AmqpAdmin amqpAdmin (ConnectionFactory connectionFactory) {
        RabbitAdmin rabbitAdmin = new RabbitAdmin(connectionFactory);

        rabbitAdmin.declareExchange(reportDirectExchange());
        rabbitAdmin.declareExchange(receiptDirectExchange());

        rabbitAdmin.declareQueue(reportQueue());
        rabbitAdmin.declareQueue(receiptQueue());

        rabbitAdmin.declareBinding(receiptBinding());
        rabbitAdmin.declareBinding(reportBinding());

        return rabbitAdmin;
    }

    /** **************************************** 回执队列相关配置 ******************************************* */

    @Bean
    public DirectExchange receiptDirectExchange () {
        return new DirectExchange(rabbitMQProperties.getE_normal_tunnel_to_csp_receipt());
    }

    @Bean
    public RabbitTemplate receiptRabbitTemplate (ConnectionFactory connectionFactory) {
        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
        rabbitTemplate.setExchange(rabbitMQProperties.getE_normal_tunnel_to_csp_receipt());
        rabbitTemplate.setConfirmCallback(new SimpleConfirmCallBack());
        rabbitTemplate.setReturnsCallback(new SimpleReturnsCallBack());
        return rabbitTemplate;
    }

    @Bean
    public Queue receiptQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_tunnel_to_csp_receipt());
    }

    @Bean
    public Binding receiptBinding () {
        return BindingBuilder.bind(receiptQueue()).to(receiptDirectExchange()).with(rabbitMQProperties.getK_normal_tunnel_to_csp_receipt());
    }

    /** **************************************** 回执队列相关配置完毕 ******************************************* */


    /** **************************************** 上报报文相关配置 ******************************************* */

    @Bean
    public DirectExchange reportDirectExchange () {
        return new DirectExchange(rabbitMQProperties.getE_normal_csp_to_tunnel_report());
    }

    @Bean
    public RabbitTemplate reportRabbitTemplate (ConnectionFactory connectionFactory) {
        RabbitTemplate rabbitTemplate = new RabbitTemplate(connectionFactory);
        rabbitTemplate.setExchange(rabbitMQProperties.getE_normal_csp_to_tunnel_report());
        rabbitTemplate.setConfirmCallback(new SimpleConfirmCallBack());
        rabbitTemplate.setReturnsCallback(new SimpleReturnsCallBack());
        return rabbitTemplate;
    }

    @Bean
    public Queue reportQueue () {
        return new Queue(rabbitMQProperties.getQ_normal_csp_to_tunnel_report());
    }

    @Bean
    public Binding reportBinding () {
        return BindingBuilder.bind(reportQueue()).to(reportDirectExchange()).with(rabbitMQProperties.getK_normal_csp_to_tunnel_report());
    }

    /** **************************************** 上报报文相关配置完成 ******************************************* */
}
