package com.emato.file.tunnel.listener;

import cn.hutool.core.convert.ConvertException;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.util.XmlUtil;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.emato.file.tunnel.config.properties.CPortProperties;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.handler.annotation.Header;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;

/**
 * @author lhm
 * @version 1.0
 * 2021-08-25 10:27
 */
@RabbitListener(
        containerFactory = "rabbitListenerContainerFactory",
        queues = {"q.normal.csp.to.tunnel.report"}
)
@Slf4j
@Component
public class ReportCEB621Listener {

    @Autowired
    private CPortProperties cPortProperties;

    /**
     * 处理上报的621报文
     * @param message   报文数据(xml字符串)
     * @param channel   信道
     */
    @RabbitHandler
    public void process (String message, Channel channel, @Header(AmqpHeaders.DELIVERY_TAG) long tag) throws IOException {
        log.info("----- 监听621报文上报, 收到消息, 开始上报! -----");
        try {
            JSONObject jsonObject = JSONUtil.parseObj(message);
            String msgFileName = jsonObject.get("msgFileName", String.class);
            String content = jsonObject.get("msgFileContent", String.class);
            String path = cPortProperties.getPushDir()  + "\\" + msgFileName;
            // 去除空格和换行符
            content = content.trim();
            XmlUtil.cleanInvalid(content);
            XmlUtil.cleanComment(content);
            File file = FileUtil.writeString(content, path, StandardCharsets.UTF_8);
            channel.basicAck(tag, false);
            log.info("----- 监听621报文上报, 文件: {} , 上报结束! -----", file.getName());
        } catch (JSONException e) {
            requeue(channel, tag);
            log.error("----- 监听621报文上报, json解析失败! 消息已重新入队! 消息体: {}, 异常: {} -----", message, e);
        } catch (IORuntimeException e) {
            requeue(channel, tag);
            log.error("----- 监听621报文上报, 写入报文文件时出现异常! 消息已重新入队! 消息体: {}, 异常: {} -----", message, e);
        } catch (ConvertException e) {
            requeue(channel, tag);
            log.error("----- 监听621报文上报, json转换失败! 消息已重新入队! 消息体: {}, 异常: {} -----", message, e);
        } catch (Exception e) {
            channel.basicAck(tag, false);
            log.error("----- 监听621报文上报, 上报出现未知异常! 消息已丢弃! 消息体: {}, 异常: {} -----", message, e);
        }
    }


    private void requeue (Channel channel, long tag) {
        try {
            channel.basicNack(tag, false, true);
        } catch (IOException ioException) {
            log.error("----- 监听621报文上报, 消息重新入队失败! -----", ioException);
            ioException.printStackTrace();
        }
    }

}
